@@ -31,10 +31,10 @@
 #include <stdlib.h>
 
 #include <bluetooth/bluetooth.h>
-#include <bluetooth/uuid.h>
 
 #include <glib.h>
 
+#include "lib/uuid.h"
 #include "att.h"
 
 const char *att_ecode2str(uint8_t status)
@@ -123,35 +123,34 @@ struct att_data_list *att_data_list_alloc(uint16_t num, uint16_t len)
 uint16_t enc_read_by_grp_req(uint16_t start, uint16_t end, bt_uuid_t *uuid,
 						uint8_t *pdu, size_t len)
 {
-	uint16_t min_len = sizeof(pdu[0]) + sizeof(start) + sizeof(end);
-	uint16_t length;
+	uint16_t uuid_len;
 
 	if (!uuid)
 		return 0;
 
 	if (uuid->type == BT_UUID16)
-		length = 2;
+		uuid_len = 2;
 	else if (uuid->type == BT_UUID128)
-		length = 16;
+		uuid_len = 16;
 	else
 		return 0;
 
-	if (len < min_len + length)
-		return 0;
-
+	/* Attribute Opcode (1 octet) */
 	pdu[0] = ATT_OP_READ_BY_GROUP_REQ;
+	/* Starting Handle (2 octets) */
 	att_put_u16(start, &pdu[1]);
+	/* Ending Handle (2 octets) */
 	att_put_u16(end, &pdu[3]);
-
+	/* Attribute Group Type (2 or 16 octet UUID) */
 	att_put_uuid(*uuid, &pdu[5]);
 
-	return min_len + length;
+	return 5 + uuid_len;
 }
 
 uint16_t dec_read_by_grp_req(const uint8_t *pdu, size_t len, uint16_t *start,
 						uint16_t *end, bt_uuid_t *uuid)
 {
-	const uint16_t min_len = sizeof(pdu[0]) + sizeof(*start) + sizeof(*end);
+	const size_t min_len = sizeof(pdu[0]) + sizeof(*start) + sizeof(*end);
 
 	if (pdu == NULL)
 		return 0;
@@ -162,7 +161,7 @@ uint16_t dec_read_by_grp_req(const uint8_t *pdu, size_t len, uint16_t *start,
 	if (pdu[0] != ATT_OP_READ_BY_GROUP_REQ)
 		return 0;
 
-	if (len < min_len + 2)
+	if (len != (min_len + 2) && len != (min_len + 16))
 		return 0;
 
 	*start = att_get_u16(&pdu[1]);
@@ -212,7 +211,25 @@ struct att_data_list *dec_read_by_grp_resp(const uint8_t *pdu, size_t len)
 	if (pdu[0] != ATT_OP_READ_BY_GROUP_RESP)
 		return NULL;
 
+	/* PDU must contain at least:
+	 * - Attribute Opcode (1 octet)
+	 * - Length (1 octet)
+	 * - Attribute Data List (at least one entry):
+	 *   - Attribute Handle (2 octets)
+	 *   - End Group Handle (2 octets)
+	 *   - Attribute Value (at least 1 octet) */
+	if (len < 7)
+		return NULL;
+
 	elen = pdu[1];
+	/* Minimum Attribute Data List size */
+	if (elen < 5)
+		return NULL;
+
+	/* Reject incomplete Attribute Data List */
+	if ((len - 2) % elen)
+		return NULL;
+
 	num = (len - 2) / elen;
 	list = att_data_list_alloc(num, elen);
 	if (list == NULL)
@@ -244,9 +261,6 @@ uint16_t enc_find_by_type_req(uint16_t start, uint16_t end, bt_uuid_t *uuid,
 	if (uuid->type != BT_UUID16)
 		return 0;
 
-	if (len < min_len)
-		return 0;
-
 	if (vlen > len - min_len)
 		vlen = len - min_len;
 
@@ -267,39 +281,27 @@ uint16_t dec_find_by_type_req(const uint8_t *pdu, size_t len, uint16_t *start,
 						uint16_t *end, bt_uuid_t *uuid,
 						uint8_t *value, size_t *vlen)
 {
-	size_t valuelen;
-	uint16_t min_len = sizeof(pdu[0]) + sizeof(*start) +
-						sizeof(*end) + sizeof(uint16_t);
-
 	if (pdu == NULL)
 		return 0;
 
-	if (len < min_len)
+	if (len < 7)
 		return 0;
 
+	/* Attribute Opcode (1 octet) */
 	if (pdu[0] != ATT_OP_FIND_BY_TYPE_REQ)
 		return 0;
 
-	/* First requested handle number */
-	if (start)
-		*start = att_get_u16(&pdu[1]);
-
-	/* Last requested handle number */
-	if (end)
-		*end = att_get_u16(&pdu[3]);
-
-	/* Always UUID16 */
-	if (uuid)
-		*uuid = att_get_uuid16(&pdu[5]);
-
-	valuelen = len - min_len;
+	/* First requested handle number (2 octets) */
+	*start = att_get_u16(&pdu[1]);
+	/* Last requested handle number (2 octets) */
+	*end = att_get_u16(&pdu[3]);
+	/* 16-bit UUID to find (2 octets) */
+	*uuid = att_get_uuid16(&pdu[5]);
 
 	/* Attribute value to find */
-	if (valuelen > 0 && value)
-		memcpy(value, pdu + min_len, valuelen);
-
-	if (vlen)
-		*vlen = valuelen;
+	*vlen = len - 7;
+	if (*vlen > 0)
+		memcpy(value, pdu + 7, *vlen);
 
 	return len;
 }
@@ -309,7 +311,7 @@ uint16_t enc_find_by_type_resp(GSList *matches, uint8_t *pdu, size_t len)
 	GSList *l;
 	uint16_t offset;
 
-	if (pdu == NULL || len < 5)
+	if (!pdu)
 		return 0;
 
 	pdu[0] = ATT_OP_FIND_BY_TYPE_RESP;
@@ -332,12 +334,21 @@ GSList *dec_find_by_type_resp(const uint8_t *pdu, size_t len)
 	GSList *matches;
 	off_t offset;
 
+	/* PDU should contain at least:
+	 * - Attribute Opcode (1 octet)
+	 * - Handles Information List (at least one entry):
+	 *   - Found Attribute Handle (2 octets)
+	 *   - Group End Handle (2 octets) */
 	if (pdu == NULL || len < 5)
 		return NULL;
 
 	if (pdu[0] != ATT_OP_FIND_BY_TYPE_RESP)
 		return NULL;
 
+	/* Reject incomplete Handles Information List */
+	if ((len - 1) % 4)
+		return NULL;
+
 	for (offset = 1, matches = NULL;
 				len >= (offset + sizeof(uint16_t) * 2);
 				offset += sizeof(uint16_t) * 2) {
@@ -354,35 +365,34 @@ GSList *dec_find_by_type_resp(const uint8_t *pdu, size_t len)
 uint16_t enc_read_by_type_req(uint16_t start, uint16_t end, bt_uuid_t *uuid,
 						uint8_t *pdu, size_t len)
 {
-	uint16_t min_len = sizeof(pdu[0]) + sizeof(start) + sizeof(end);
-	uint16_t length;
+	uint16_t uuid_len;
 
 	if (!uuid)
 		return 0;
 
 	if (uuid->type == BT_UUID16)
-		length = 2;
+		uuid_len = 2;
 	else if (uuid->type == BT_UUID128)
-		length = 16;
+		uuid_len = 16;
 	else
 		return 0;
 
-	if (len < min_len + length)
-		return 0;
-
+	/* Attribute Opcode (1 octet) */
 	pdu[0] = ATT_OP_READ_BY_TYPE_REQ;
+	/* Starting Handle (2 octets) */
 	att_put_u16(start, &pdu[1]);
+	/* Ending Handle (2 octets) */
 	att_put_u16(end, &pdu[3]);
-
+	/* Attribute Type (2 or 16 octet UUID) */
 	att_put_uuid(*uuid, &pdu[5]);
 
-	return min_len + length;
+	return 5 + uuid_len;
 }
 
 uint16_t dec_read_by_type_req(const uint8_t *pdu, size_t len, uint16_t *start,
 						uint16_t *end, bt_uuid_t *uuid)
 {
-	const uint16_t min_len = sizeof(pdu[0]) + sizeof(*start) + sizeof(*end);
+	const size_t min_len = sizeof(pdu[0]) + sizeof(*start) + sizeof(*end);
 
 	if (pdu == NULL)
 		return 0;
@@ -390,7 +400,7 @@ uint16_t dec_read_by_type_req(const uint8_t *pdu, size_t len, uint16_t *start,
 	if (start == NULL || end == NULL || uuid == NULL)
 		return 0;
 
-	if (len < min_len + 2)
+	if (len != (min_len + 2) && len != (min_len + 16))
 		return 0;
 
 	if (pdu[0] != ATT_OP_READ_BY_TYPE_REQ)
@@ -444,7 +454,24 @@ struct att_data_list *dec_read_by_type_resp(const uint8_t *pdu, size_t len)
 	if (pdu[0] != ATT_OP_READ_BY_TYPE_RESP)
 		return NULL;
 
+	/* PDU must contain at least:
+	 * - Attribute Opcode (1 octet)
+	 * - Length (1 octet)
+	 * - Attribute Data List (at least one entry):
+	 *   - Attribute Handle (2 octets)
+	 *   - Attribute Value (at least 1 octet) */
+	if (len < 5)
+		return NULL;
+
 	elen = pdu[1];
+	/* Minimum Attribute Data List size */
+	if (elen < 3)
+		return NULL;
+
+	/* Reject incomplete Attribute Data List */
+	if ((len - 2) % elen)
+		return NULL;
+
 	num = (len - 2) / elen;
 	list = att_data_list_alloc(num, elen);
 	if (list == NULL)
@@ -468,9 +495,6 @@ uint16_t enc_write_cmd(uint16_t handle, const uint8_t *value, size_t vlen,
 	if (pdu == NULL)
 		return 0;
 
-	if (len < min_len)
-		return 0;
-
 	if (vlen > len - min_len)
 		vlen = len - min_len;
 
@@ -517,9 +541,6 @@ uint16_t enc_write_req(uint16_t handle, const uint8_t *value, size_t vlen,
 	if (pdu == NULL)
 		return 0;
 
-	if (len < min_len)
-		return 0;
-
 	if (vlen > len - min_len)
 		vlen = len - min_len;
 
@@ -559,7 +580,7 @@ uint16_t dec_write_req(const uint8_t *pdu, size_t len, uint16_t *handle,
 	return len;
 }
 
-uint16_t enc_write_resp(uint8_t *pdu, size_t len)
+uint16_t enc_write_resp(uint8_t *pdu)
 {
 	if (pdu == NULL)
 		return 0;
@@ -582,37 +603,31 @@ uint16_t dec_write_resp(const uint8_t *pdu, size_t len)
 
 uint16_t enc_read_req(uint16_t handle, uint8_t *pdu, size_t len)
 {
-	const uint16_t min_len = sizeof(pdu[0]) + sizeof(handle);
-
 	if (pdu == NULL)
 		return 0;
 
-	if (len < min_len)
-		return 0;
-
+	/* Attribute Opcode (1 octet) */
 	pdu[0] = ATT_OP_READ_REQ;
+	/* Attribute Handle (2 octets) */
 	att_put_u16(handle, &pdu[1]);
 
-	return min_len;
+	return 3;
 }
 
 uint16_t enc_read_blob_req(uint16_t handle, uint16_t offset, uint8_t *pdu,
-									size_t len)
+								size_t len)
 {
-	const uint16_t min_len = sizeof(pdu[0]) + sizeof(handle) +
-							sizeof(offset);
-
 	if (pdu == NULL)
 		return 0;
 
-	if (len < min_len)
-		return 0;
-
+	/* Attribute Opcode (1 octet) */
 	pdu[0] = ATT_OP_READ_BLOB_REQ;
+	/* Attribute Handle (2 octets) */
 	att_put_u16(handle, &pdu[1]);
+	/* Value Offset (2 octets) */
 	att_put_u16(offset, &pdu[3]);
 
-	return min_len;
+	return 5;
 }
 
 uint16_t dec_read_req(const uint8_t *pdu, size_t len, uint16_t *handle)
@@ -682,7 +697,7 @@ uint16_t enc_read_resp(uint8_t *value, size_t vlen, uint8_t *pdu, size_t len)
 }
 
 uint16_t enc_read_blob_resp(uint8_t *value, size_t vlen, uint16_t offset,
-							uint8_t *pdu, size_t len)
+						uint8_t *pdu, size_t len)
 {
 	if (pdu == NULL)
 		return 0;
@@ -698,17 +713,18 @@ uint16_t enc_read_blob_resp(uint8_t *value, size_t vlen, uint16_t offset,
 	return vlen + 1;
 }
 
-ssize_t dec_read_resp(const uint8_t *pdu, size_t len, uint8_t *value, size_t vlen)
+ssize_t dec_read_resp(const uint8_t *pdu, size_t len, uint8_t *value,
+								size_t vlen)
 {
 	if (pdu == NULL)
 		return -EINVAL;
 
-	if (value == NULL)
-		return -EINVAL;
-
 	if (pdu[0] != ATT_OP_READ_RESP)
 		return -EINVAL;
 
+	if (value == NULL)
+		return len - 1;
+
 	if (vlen < (len - 1))
 		return -ENOBUFS;
 
@@ -718,39 +734,34 @@ ssize_t dec_read_resp(const uint8_t *pdu, size_t len, uint8_t *value, size_t vle
 }
 
 uint16_t enc_error_resp(uint8_t opcode, uint16_t handle, uint8_t status,
-							uint8_t *pdu, size_t len)
+						uint8_t *pdu, size_t len)
 {
-	const uint16_t min_len = sizeof(pdu[0]) + sizeof(opcode) +
-						sizeof(handle) + sizeof(status);
-	uint16_t u16;
-
-	if (len < min_len)
-		return 0;
-
-	u16 = htobs(handle);
+	/* Attribute Opcode (1 octet) */
 	pdu[0] = ATT_OP_ERROR;
+	/* Request Opcode In Error (1 octet) */
 	pdu[1] = opcode;
-	memcpy(&pdu[2], &u16, sizeof(u16));
+	/* Attribute Handle In Error (2 octets) */
+	att_put_u16(handle, &pdu[2]);
+	/* Error Code (1 octet) */
 	pdu[4] = status;
 
-	return min_len;
+	return 5;
 }
 
-uint16_t enc_find_info_req(uint16_t start, uint16_t end, uint8_t *pdu, size_t len)
+uint16_t enc_find_info_req(uint16_t start, uint16_t end, uint8_t *pdu,
+								size_t len)
 {
-	const uint16_t min_len = sizeof(pdu[0]) + sizeof(start) + sizeof(end);
-
 	if (pdu == NULL)
 		return 0;
 
-	if (len < min_len)
-		return 0;
-
+	/* Attribute Opcode (1 octet) */
 	pdu[0] = ATT_OP_FIND_INFO_REQ;
+	/* Starting Handle (2 octets) */
 	att_put_u16(start, &pdu[1]);
+	/* Ending Handle (2 octets) */
 	att_put_u16(end, &pdu[3]);
 
-	return min_len;
+	return 5;
 }
 
 uint16_t dec_find_info_req(const uint8_t *pdu, size_t len, uint16_t *start,
@@ -777,7 +788,7 @@ uint16_t dec_find_info_req(const uint8_t *pdu, size_t len, uint16_t *start,
 }
 
 uint16_t enc_find_info_resp(uint8_t format, struct att_data_list *list,
-							uint8_t *pdu, size_t len)
+						uint8_t *pdu, size_t len)
 {
 	uint8_t *ptr;
 	size_t i, w;
@@ -907,33 +918,26 @@ uint16_t dec_indication(const uint8_t *pdu, size_t len, uint16_t *handle,
 
 uint16_t enc_confirmation(uint8_t *pdu, size_t len)
 {
-	const uint16_t min_len = sizeof(pdu[0]);
-
 	if (pdu == NULL)
 		return 0;
 
-	if (len < min_len)
-		return 0;
-
+	/* Attribute Opcode (1 octet) */
 	pdu[0] = ATT_OP_HANDLE_CNF;
 
-	return min_len;
+	return 1;
 }
 
 uint16_t enc_mtu_req(uint16_t mtu, uint8_t *pdu, size_t len)
 {
-	const uint16_t min_len = sizeof(pdu[0]) + sizeof(mtu);
-
 	if (pdu == NULL)
 		return 0;
 
-	if (len < min_len)
-		return 0;
-
+	/* Attribute Opcode (1 octet) */
 	pdu[0] = ATT_OP_MTU_REQ;
+	/* Client Rx MTU (2 octets) */
 	att_put_u16(mtu, &pdu[1]);
 
-	return min_len;
+	return 3;
 }
 
 uint16_t dec_mtu_req(const uint8_t *pdu, size_t len, uint16_t *mtu)
@@ -959,18 +963,15 @@ uint16_t dec_mtu_req(const uint8_t *pdu, size_t len, uint16_t *mtu)
 
 uint16_t enc_mtu_resp(uint16_t mtu, uint8_t *pdu, size_t len)
 {
-	const uint16_t min_len = sizeof(pdu[0]) + sizeof(mtu);
-
 	if (pdu == NULL)
 		return 0;
 
-	if (len < min_len)
-		return 0;
-
+	/* Attribute Opcode (1 octet) */
 	pdu[0] = ATT_OP_MTU_RESP;
+	/* Server Rx MTU (2 octets) */
 	att_put_u16(mtu, &pdu[1]);
 
-	return min_len;
+	return 3;
 }
 
 uint16_t dec_mtu_resp(const uint8_t *pdu, size_t len, uint16_t *mtu)
@@ -995,7 +996,8 @@ uint16_t dec_mtu_resp(const uint8_t *pdu, size_t len, uint16_t *mtu)
 }
 
 uint16_t enc_prep_write_req(uint16_t handle, uint16_t offset,
-			const uint8_t *value, size_t vlen, uint8_t *pdu, size_t len)
+					const uint8_t *value, size_t vlen,
+					uint8_t *pdu, size_t len)
 {
 	const uint16_t min_len = sizeof(pdu[0]) + sizeof(handle) +
 								sizeof(offset);
@@ -1003,13 +1005,63 @@ uint16_t enc_prep_write_req(uint16_t handle, uint16_t offset,
 	if (pdu == NULL)
 		return 0;
 
+	if (vlen > len - min_len)
+		vlen = len - min_len;
+
+	pdu[0] = ATT_OP_PREP_WRITE_REQ;
+	att_put_u16(handle, &pdu[1]);
+	att_put_u16(offset, &pdu[3]);
+
+	if (vlen > 0) {
+		memcpy(&pdu[5], value, vlen);
+		return min_len + vlen;
+	}
+
+	return min_len;
+}
+
+uint16_t dec_prep_write_req(const uint8_t *pdu, size_t len, uint16_t *handle,
+				uint16_t *offset, uint8_t *value, size_t *vlen)
+{
+	const uint16_t min_len = sizeof(pdu[0]) + sizeof(*handle) +
+							sizeof(*offset);
+
+	if (pdu == NULL)
+		return 0;
+
+	if (handle == NULL || offset == NULL || value == NULL || vlen == NULL)
+		return 0;
+
 	if (len < min_len)
 		return 0;
 
+	if (pdu[0] != ATT_OP_PREP_WRITE_REQ)
+		return 0;
+
+	*handle = att_get_u16(&pdu[1]);
+	*offset = att_get_u16(&pdu[3]);
+
+	*vlen = len - min_len;
+	if (*vlen > 0)
+		memcpy(value, pdu + min_len, *vlen);
+
+	return len;
+}
+
+uint16_t enc_prep_write_resp(uint16_t handle, uint16_t offset,
+					const uint8_t *value, size_t vlen,
+					uint8_t *pdu, size_t len)
+{
+	const uint16_t min_len = sizeof(pdu[0]) + sizeof(handle) +
+								sizeof(offset);
+
+	if (pdu == NULL)
+		return 0;
+
 	if (vlen > len - min_len)
 		vlen = len - min_len;
 
-	pdu[0] = ATT_OP_PREP_WRITE_REQ;
+	pdu[0] = ATT_OP_PREP_WRITE_RESP;
 	att_put_u16(handle, &pdu[1]);
 	att_put_u16(offset, &pdu[3]);
 
@@ -1050,23 +1102,52 @@ uint16_t dec_prep_write_resp(const uint8_t *pdu, size_t len, uint16_t *handle,
 
 uint16_t enc_exec_write_req(uint8_t flags, uint8_t *pdu, size_t len)
 {
-	const uint16_t min_len = sizeof(pdu[0]) + sizeof(flags);
-
 	if (pdu == NULL)
 		return 0;
 
-	if (len < min_len)
-		return 0;
-
 	if (flags > 1)
 		return 0;
 
+	/* Attribute Opcode (1 octet) */
 	pdu[0] = ATT_OP_EXEC_WRITE_REQ;
+	/* Flags (1 octet) */
 	pdu[1] = flags;
 
+	return 2;
+}
+
+uint16_t dec_exec_write_req(const uint8_t *pdu, size_t len, uint8_t *flags)
+{
+	const uint16_t min_len = sizeof(pdu[0]) + sizeof(*flags);
+
+	if (pdu == NULL)
+		return 0;
+
+	if (flags == NULL)
+		return 0;
+
+	if (len < min_len)
+		return 0;
+
+	if (pdu[0] != ATT_OP_EXEC_WRITE_REQ)
+		return 0;
+
+	*flags = pdu[1];
+
 	return min_len;
 }
 
+uint16_t enc_exec_write_resp(uint8_t *pdu)
+{
+	if (pdu == NULL)
+		return 0;
+
+	/* Attribute Opcode (1 octet) */
+	pdu[0] = ATT_OP_EXEC_WRITE_RESP;
+
+	return 1;
+}
+
 uint16_t dec_exec_write_resp(const uint8_t *pdu, size_t len)
 {
 	const uint16_t min_len = sizeof(pdu[0]);
