@@ -39,7 +39,7 @@
 #include <readline/history.h>
 
 #include "lib/uuid.h"
-#include <btio/btio.h>
+#include "btio/btio.h"
 #include "att.h"
 #include "gattrib.h"
 #include "gatt.h"
