@@ -150,8 +150,9 @@ extern "C" {
 #define HDP_SVCLASS_ID			0x1400
 #define HDP_SOURCE_SVCLASS_ID		0x1401
 #define HDP_SINK_SVCLASS_ID		0x1402
-#define APPLE_AGENT_SVCLASS_ID		0x2112
+#define GENERIC_ACCESS_SVCLASS_ID	0x1800
 #define GENERIC_ATTRIB_SVCLASS_ID	0x1801
+#define APPLE_AGENT_SVCLASS_ID		0x2112
 
 /*
  * Standard profile descriptor identifiers; note these
@@ -224,9 +225,9 @@ extern "C" {
 #define HDP_PROFILE_ID			HDP_SVCLASS_ID
 #define HDP_SOURCE_PROFILE_ID		HDP_SOURCE_SVCLASS_ID
 #define HDP_SINK_PROFILE_ID		HDP_SINK_SVCLASS_ID
-#define APPLE_AGENT_PROFILE_ID		APPLE_AGENT_SVCLASS_ID
-#define GENERIC_ACCESS_PROFILE_ID	0x1800
+#define GENERIC_ACCESS_PROFILE_ID	GENERIC_ACCESS_SVCLASS_ID
 #define GENERIC_ATTRIB_PROFILE_ID	GENERIC_ATTRIB_SVCLASS_ID
+#define APPLE_AGENT_PROFILE_ID		APPLE_AGENT_SVCLASS_ID
 
 /*
  * Compatibility macros for the old MDP acronym
